package com.ruoyi.common.utils;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.common.CustomMultipartFile;
import com.ruoyi.common.config.RuoYiConfig;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.utils.file.FileUploadUtils;
import com.ruoyi.project.domain.DataSource;
import com.ruoyi.project.domain.Scene;
import com.ruoyi.project.domain.sourceData.ImageData;
import com.ruoyi.project.domain.sourceData.VideoData;
import com.ruoyi.project.domain.vo.DataSourceVo;
import com.ruoyi.project.mapper.DataSourceMapper;
import com.ruoyi.project.mapper.ImageDataMapper;
import com.ruoyi.project.mapper.VideoDataMapper;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Objects;

public class CommonUtils {

    /**
     * 通过url获取json文件内容
     */
    public static <T> T getContentByUrl(String url, Class<T> clazz) {
        RestTemplate restTemplate = new RestTemplate();
        HttpHeaders headers = new HttpHeaders();
        HttpEntity<T> entity = new HttpEntity<>(headers);

        ResponseEntity<T> response = restTemplate.exchange(url, HttpMethod.GET, entity, clazz);

        return Objects.requireNonNull(response.getBody());
    }

    /**
     * 新增spot文件
     */
    public static AjaxResult createSpotFile(JSONObject spotJSON) {
        // 将 spotStr 写入临时 JSON 文件
        File tempFile = null;
        try {
            tempFile = File.createTempFile("spot", ".json");
            try (FileWriter fileWriter = new FileWriter(tempFile)) {
                fileWriter.write(spotJSON.toJSONString());
            }

            // 使用自定义的 CustomMultipartFile
            MultipartFile multipartFile = new CustomMultipartFile(tempFile);
            String uploadedUrl;
            try {
                uploadedUrl = uploadSpotFile(multipartFile);
                return AjaxResult.success("上传文件成功", uploadedUrl);
            } catch (Exception e) {
                return AjaxResult.error("上传文件失败");
            }
        } catch (IOException e) {
            return AjaxResult.error("创建临时文件失败");
        } finally {
            // 删除临时文件
            if (tempFile != null) {
                boolean isDeleted = tempFile.delete();
                if (!isDeleted) {
                    System.out.println("删除临时文件失败，文件路径: " + tempFile.getAbsolutePath());
                }
            }
        }
    }

    /**
     * 上传spot文件
     */
    public static String uploadSpotFile(MultipartFile file) throws Exception {
        // 上传文件路径
        String filePath = RuoYiConfig.getUploadPath() + "/spotFile";
        // 上传并返回新文件名称
        return FileUploadUtils.upload(filePath, file);
    }

    /**
     * 删除spot文件
     *
     * @param spotPath spot文件路径
     */
    public static void deleteSpotFile(String spotPath) {
        // 删除文件
        String filePath = RuoYiConfig.getUploadPath();

        //由于在RuoYiConfig.getUploadPath()中带有upload导致文件搜索失败，所以去除
        String keyword = "/upload";
        int endIndex = filePath.indexOf(keyword);
        if (endIndex != -1) {
            filePath = filePath.substring(0, (filePath.length() - 7));
        }
        // 拼接文件路径
        String deletePath = filePath + StringUtils.substringAfter(spotPath, Constants.RESOURCE_PREFIX);
        // 删除旧文件
        deleteFile(deletePath);
    }

    /**
     * 删除指定路径的文件
     */
    public static void deleteFile(String filePath) {
        File file = new File(filePath);
        if (file.exists()) {
            boolean isDeleted = file.delete();
            if (isDeleted) {
                System.out.println("文件删除成功，文件路径: " + filePath);
            } else {
                System.out.println("文件删除失败，文件路径: " + filePath);
            }
        } else {
            System.out.println("文件不存在，文件路径: " + filePath);
        }
    }


    /**
     * 将<T>str转化为<T>List
     */
    public static <T> List<T> translateToList(String jsonStr, String key, Class<T> clazz) {
        JSONObject jsonObj = JSONObject.parseObject(jsonStr);
        String data = jsonObj.getString(key);
        return JSON.parseArray(data, clazz);
    }

    /**
     * 获取数据源voMap，主键为场景与展示点拼接的字符串
     *
     * @return HashMap<String, DataSourceVo>
     */
    public static HashMap<String, DataSourceVo> getDataSourceVoMap(Scene scene, DataSourceMapper dataSourceMapper,
                                                                   ImageDataMapper imageDataMapper,
                                                                   VideoDataMapper videoDataMapper) {
        List<DataSource> dataSources = dataSourceMapper.selectDataSourceListByAppId(scene.getAppId());
        HashMap<String, DataSourceVo> dataMap = new HashMap<>(); // 用于储存dataSourceVo，避免多次请求数据库
        for (DataSource dataSource : dataSources) {
            if (!StringUtils.isEmpty(dataSource.getSpot())) {
                // 根据数据类型来封装dataSourceVo
                DataSourceVo dataSourceVo = new DataSourceVo();
                if (dataSource.getType() == 1) {
                    ImageData imageData = imageDataMapper.selectImageDataByDataSourceId(dataSource.getId());
                    dataSourceVo = new DataSourceVo(dataSource, imageData.getUrl(), imageData.getCover());
                } else if (dataSource.getType() == 2) {
                    VideoData videoData = videoDataMapper.selectVideoDataByDataSourceId(dataSource.getId());
                    dataSourceVo = new DataSourceVo(dataSource, videoData.getUrl(), videoData.getCover());
                }

                String spot = dataSource.getSpot();

                // 判断展示点分别挂载了哪些数据源
                if (spot.split("#").length == 0 && StringUtils.isEmpty(spot)) {
                    dataMap.put(dataSource.getSpot(), dataSourceVo);
                } else {
                    String[] spotTitles = dataSource.getSpot().split("#");
                    for (String s : spotTitles) {
                        dataMap.put(s, dataSourceVo);
                    }
                }
            }
        }

        return dataMap;
    }
}
