package com.ruoyi.cultural.service.impl;


import java.util.ArrayList;
import java.util.List;

import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.QRCodeUtil;
import com.ruoyi.cultural.domain.Cultural;
import com.ruoyi.cultural.domain.CulturalLog;
import com.ruoyi.cultural.domain.CulturalQrCode;
import com.ruoyi.cultural.domain.CulturalType;
import com.ruoyi.project.domain.App;
import com.ruoyi.cultural.domain.vo.CulturalVo;
import com.ruoyi.project.mapper.AppMapper;
import com.ruoyi.cultural.mapper.CulturalLogMapper;
import com.ruoyi.cultural.mapper.CulturalTypeMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.cultural.mapper.CulturalMapper;
import com.ruoyi.cultural.service.ICulturalService;

/**
 * 文创产品Service业务层处理
 *
 * @author seeseele
 * @date 2024-07-19
 */
@Service("culturalServiceImplement")
public class CulturalServiceImpl implements ICulturalService {

    private final CulturalMapper culturalMapper;

    private final CulturalTypeMapper culturalTypeMapper;

    private final CulturalLogMapper culturalLogMapper;

    private final AppMapper appMapper;

    public CulturalServiceImpl(@Autowired CulturalMapper culturalMapper,
                               @Autowired CulturalTypeMapper culturalTypeMapper,
                               @Autowired CulturalLogMapper culturalLogMapper,
                               @Autowired AppMapper appMapper) {
        this.culturalMapper = culturalMapper;
        this.culturalTypeMapper = culturalTypeMapper;
        this.culturalLogMapper = culturalLogMapper;
        this.appMapper = appMapper;
    }

    public CulturalVo translateToVo(Cultural cultural) {
        App app = appMapper.selectAppById(cultural.getAppId());
        CulturalType culturalType = culturalTypeMapper.selectCulturalTypeById(cultural.getTypeId());

        CulturalVo culturalVo = new CulturalVo(cultural);
        culturalVo.setApp(app);
        culturalVo.setType(culturalType);

        return culturalVo;
    }

    /**
     * 查询文创产品
     *
     * @param id 文创产品主键
     * @return 文创产品
     */
    @Override
    public CulturalVo selectCulturalById(Long id) {
        return translateToVo(culturalMapper.selectCulturalById(id));
    }

    /**
     * 查询文创产品列表
     *
     * @param cultural 文创产品
     * @return 文创产品
     */
    @Override
    public List<CulturalVo> selectCulturalList(Cultural cultural) {
        List<Cultural> culturalList = culturalMapper.selectCulturalList(cultural);
        List<CulturalVo> voList = new ArrayList<>();
        for (Cultural c : culturalList) {
            voList.add(translateToVo(c));
        }
        return voList;
    }

    /**
     * 查询所有文创产品列表
     *
     * @param cultural 文创产品
     * @return 文创产品集合
     */
    @Override
    public List<CulturalVo> selectCulturalAllList(Cultural cultural) {
        List<Cultural> culturalList = culturalMapper.selectCulturalAllList(cultural);
        List<CulturalVo> voList = new ArrayList<>();
        for (Cultural c : culturalList) {
            voList.add(translateToVo(c));
        }
        return voList;
    }

    /**
     * 获取要生成二维码的内容
     */
    public String getQrCodeContent(Cultural cultural) {
        return "http://img.han-zhi.net/wlcd/pv/index.html";
    }

    /**
     * 获取文创产品二维码列表
     *
     * @param ids 需要获取的二维码的文创产品主键集合
     * @return 文创产品集合
     */
    @Override
    public List<CulturalQrCode> getQrCodeList(Long[] ids) {
        List<CulturalQrCode> list = new ArrayList<>();
        for (Long id : ids) {
            Cultural cultural = culturalMapper.selectCulturalById(id);
            if (cultural == null) {
                continue;
            }
            String content = getQrCodeContent(cultural);
            byte[] qrCode = QRCodeUtil.createCodeToByteArray(content);

            CulturalQrCode culturalQrCode = new CulturalQrCode();
            culturalQrCode.setTitle(cultural.getTitle());
            culturalQrCode.setQrCode(qrCode);

            list.add(culturalQrCode);
        }

        return list;
    }

    /**
     * 新增文创产品
     *
     * @param cultural 文创产品
     * @return 结果
     */
    @Override
    public boolean insertCultural(Cultural cultural) {
        Long userId = SecurityUtils.getUserId();
        cultural.setUserId(userId);
        cultural.setCreateTime(DateUtils.getNowDate());

        boolean flag = culturalMapper.insertCultural(cultural) > 0;

        CulturalLog culturalLog = culturalLogMapper.selectCulturalLogByUserIdAndTypeId(userId, cultural.getTypeId());
        if (culturalLog == null) {
            culturalLog = new CulturalLog();
            culturalLog.setUserId(userId);
            culturalLog.setTypeId(cultural.getTypeId());
            culturalLog.setCount(1L);
            culturalLog.setCreateTime(DateUtils.getNowDate());
            culturalLog.setCreateBy(SecurityUtils.getUsername());
            culturalLogMapper.insertCulturalLog(culturalLog);
        } else {
            culturalLog.setCount(culturalLog.getCount() + 1L);
            culturalLog.setUpdateTime(DateUtils.getNowDate());
            culturalLog.setUpdateBy(SecurityUtils.getUsername());
            culturalLogMapper.updateCulturalLog(culturalLog);
        }

        return flag;
    }

    /**
     * 修改文创产品
     *
     * @param cultural 文创产品
     * @return 结果
     */
    @Override
    public boolean updateCultural(Cultural cultural) {
        cultural.setUpdateBy(SecurityUtils.getUsername());
        cultural.setUpdateTime(DateUtils.getNowDate());
        return culturalMapper.updateCultural(cultural) > 0;
    }

    /**
     * 批量删除文创产品
     *
     * @param ids 需要删除的文创产品主键
     * @return 结果
     */
    @Override
    public boolean deleteCulturalByIds(Long[] ids) {
        return culturalMapper.deleteCulturalByIds(ids) > 0;
    }

    /**
     * 删除文创产品信息
     *
     * @param id 文创产品主键
     * @return 结果
     */
    @Override
    public boolean deleteCulturalById(Long id) {
        return culturalMapper.deleteCulturalById(id) > 0;
    }

}
