package com.ruoyi.project.controller;

import cn.dev33.satoken.stp.SaTokenInfo;
import cn.dev33.satoken.stp.StpUtil;
import cn.dev33.satoken.util.SaResult;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.common.annotation.Anonymous;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.project.domain.OrderDetails;
import com.ruoyi.project.domain.Orders;
import com.ruoyi.project.domain.Template;
import com.ruoyi.project.domain.vo.OrderVo;
import com.ruoyi.project.service.IOrdersService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 订单Controller
 *
 * @author seeseele
 * @date 2025-02-26
 */
@Api(tags = "订单模块")
@RestController
@RequestMapping("/vrone/backend/orders")
public class OrdersController extends BaseController {

    private final IOrdersService ordersService;

    public OrdersController(
            @Qualifier("ordersServiceImplement") IOrdersService ordersService) {
        this.ordersService = ordersService;
    }

    @Anonymous
    @GetMapping("/listByUserId")
    @ApiOperation(value = "通过userId查询订单列表", notes = "携带token调用")
    public AjaxResult listByUserId() {
        // 通过sa-token获取用户id
        long userId = StpUtil.getLoginIdAsLong();

        List<OrderVo> orderVoList = ordersService.selectOrderVoListByUserId(userId);

        return success(orderVoList);
    }

    @Anonymous
    @GetMapping(value = "/{id}")
    @ApiOperation(value = "获取订单详细信息", notes = "携带token调用,传参为id（订单id）")
    public AjaxResult getInfo(
            @PathVariable("id") Long id) {
        long userId = StpUtil.getLoginIdAsLong();

        return success(ordersService.selectOrdersToCache(userId, id));
    }

    @Anonymous
    @PostMapping("/generate")
    @ApiOperation(value = "生成订单", notes = "传参为jsonObject，其中包含token，orders(订单)，detailList(订单详情列表)")
    public AjaxResult generate(
            @RequestBody JSONObject jsonObject) {
        // 获取订单信息
        Orders orders = jsonObject.getObject("orders", Orders.class);
        // 获取订单详情str
        String detailStr = jsonObject.getString("detailList");
        // 转化为订单详情列表
        List<OrderDetails> detailList = JSON.parseArray(detailStr, OrderDetails.class);

        // 通过sa-token获取用户id
        long userId = StpUtil.getLoginIdAsLong();
        if (userId != orders.getUserId()) {
            return error("生成订单用户和订单用户不同");
        }

        for (OrderDetails orderDetails : detailList) {
            Template template = ordersService.selectTemplateById(orderDetails.getTemplateId());
            if (template == null) {
                return error("模板不存在");
            }
            if (template.getStatus() != 1) {
                return error("模板状态异常");
            }
            if (template.getPrice().compareTo(orderDetails.getPrice()) != 0) {
                return error("订单价格异常");
            }
        }

        OrderVo orderVo = ordersService.createOrderVo(orders, detailList);

        AjaxResult ajaxResult = new AjaxResult();
        ajaxResult.put("code", 200);
        ajaxResult.put("msg", "生成订单成功");
        ajaxResult.put("data", orderVo);
        return ajaxResult;
    }

    @Anonymous
    @PutMapping("/payment")
    @ApiOperation(value = "支付订单", notes = "传参为jsonObject，其中包含token，订单")
    public AjaxResult payment(
            @RequestBody JSONObject jsonObject) {
        Long orderId = jsonObject.getLong("orderId");

        OrderVo orderVo = ordersService.selectOrdersToCache(StpUtil.getLoginIdAsLong(), orderId);
        if (orderVo == null) {
            return error("订单不存在");
        }

        return ordersService.payment(StpUtil.getLoginIdAsLong(), orderId);
    }

    @Anonymous
    @PutMapping("/cancel")
    @ApiOperation(value = "取消订单", notes = "传参为jsonObject，其中包含token，订单id")
    public AjaxResult cancel(
            @RequestBody JSONObject jsonObject) {
        Long orderId = jsonObject.getLong("orderId");

        OrderVo orderVo = ordersService.selectOrdersToCache(StpUtil.getLoginIdAsLong(), orderId);
        if (orderVo == null) {
            return error("订单不存在");
        }

        return ordersService.cancelOrder(StpUtil.getLoginIdAsLong(), orderId);
    }

    @Anonymous
    @PutMapping("/refund")
    @ApiOperation(value = "订单退款", notes = "传参为jsonObject，其中包含token，订单id")
    public AjaxResult refund(
            @RequestBody JSONObject jsonObject) {
        Long orderId = jsonObject.getLong("orderId");

        OrderVo orderVo = ordersService.selectOrdersToCache(StpUtil.getLoginIdAsLong(), orderId);
        if (orderVo == null) {
            return error("订单不存在");
        }

        return ordersService.refundOrder(StpUtil.getLoginIdAsLong(), orderId);
    }

}
