package com.ruoyi.project.service.cache;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.common.constant.CacheConstants;
import com.ruoyi.common.constant.OrderConst;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.project.domain.vo.OrderVo;
import org.redisson.api.RBlockingDeque;
import org.redisson.api.RDelayedQueue;
import org.redisson.api.RedissonClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Collection;
import java.util.Map;
import java.util.concurrent.TimeUnit;

@Component
public class OrderCacheService {

    private static final Logger logger = LoggerFactory.getLogger("orderCacheService");

    private final RedisCache redisCache;

    private final RedissonClient redissonClient;

    public OrderCacheService(
            @Autowired RedisCache redisCache,
            @Autowired RedissonClient redissonClient) {
        this.redisCache = redisCache;
        this.redissonClient = redissonClient;
    }

    /**
     * 获取订单vo缓存key
     *
     * @param userId   用户id
     * @param ordersId 订单id
     * @return 缓存id
     */
    public String getCacheKey(Long userId, Long ordersId) {
        return CacheConstants.VRONE_ORDER_KEY + userId + "&" + ordersId;
    }

    /**
     * 通过用户id和订单id从缓存中查询订单
     */
    public OrderVo selectOrderVoToCache(Long userId, Long ordersId) {
        String cacheKey = getCacheKey(userId, ordersId);
        return selectOrderVoToCache(cacheKey);
    }

    /**
     * 通过命名空间获取缓存中的所有订单数据
     */
    public Map<String, Object> getNamespaceData() {
        return redisCache.getAllDataInNamespace(CacheConstants.VRONE_ORDER_KEY);
    }

    /**
     * 通过key从缓存中查询订单
     */
    public OrderVo selectOrderVoToCache(String cacheKey) {
        // 从缓存中获取 JSONObject
        JSONObject orderVoJson = redisCache.getCacheObject(cacheKey);
        if (orderVoJson == null) {
            System.out.println("订单 " + cacheKey + " 在缓存中不存在");
            return null;
        }
        // 将 JSONObject 转换为 OrderVo 对象
        return JSON.parseObject(orderVoJson.toJSONString(), OrderVo.class);
    }

    /**
     * 向缓存中添加订单vo
     */
    public void addOrderToCache(OrderVo orderVo) {
        String cacheKey = getCacheKey(orderVo.getUserId(), orderVo.getId());
        // 使用 FastJSON2 序列化 OrderVo 对象
        String orderVoJsonString = JSON.toJSONString(orderVo);
        JSONObject orderVoJson = JSON.parseObject(orderVoJsonString);

        redisCache.setCacheObject(cacheKey, orderVoJson);
    }

    /**
     * 将创建订单放入延迟队列
     */
    public void insertOrderToQueue(OrderVo orderVo) {
        String cacheKey = getCacheKey(orderVo.getUserId(), orderVo.getId());

        // 获取普通队列
        RBlockingDeque<String> blockingDeque = redissonClient.getBlockingDeque(OrderConst.CANCEL_QUEUE_KEY);
        // 获取基于这个队列的延迟队列
        RDelayedQueue<String> delayedQueue = redissonClient.getDelayedQueue(blockingDeque);

        // 将订单ID放入延迟队列，设置延迟时间为10分钟
        long delay = TimeUnit.MINUTES.toMillis(10);  // 10分钟
        delayedQueue.offer(cacheKey, delay, TimeUnit.MILLISECONDS);
        // 指定日期格式
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        // 打印添加任务时间
        logger.info("添加时间：{}", formatter.format(LocalDateTime.now()));
        // 可以保存订单信息到数据库
        logger.info("订单 {} 创建成功，10分钟后自动取消.", cacheKey);
    }

    /**
     * 检查指定 cacheKey 是否存在于延迟队列或阻塞队列中
     *
     * @param cacheKey 缓存键
     * @return 如果存在于任意队列中返回 true，否则返回 false
     */
    public boolean isOrderInQueues(String cacheKey) {
        // 获取阻塞队列
        RBlockingDeque<String> blockingDeque = redissonClient.getBlockingDeque(OrderConst.CANCEL_QUEUE_KEY);
        // 获取基于阻塞队列的延迟队列
        RDelayedQueue<String> delayedQueue = redissonClient.getDelayedQueue(blockingDeque);

        // 检查是否存在于阻塞队列中
        boolean existsInBlockingDeque = blockingDeque.contains(cacheKey);
        // 检查是否存在于延迟队列中
        boolean existsInDelayedQueue = delayedQueue.contains(cacheKey);

        return existsInDelayedQueue || existsInBlockingDeque;
    }


    /**
     * 手动移除延迟队列中的订单
     */
    public void removeOrderToQueue(String cacheKey) {
        RBlockingDeque<String> blockedDeque = redissonClient.getBlockingDeque(OrderConst.CANCEL_QUEUE_KEY);
        RDelayedQueue<String> delayedQueue = redissonClient.getDelayedQueue(blockedDeque);

        // 从延时队列中移除
        boolean removedFromDelayedQueue = delayedQueue.remove(cacheKey);
        // 从阻塞队列中移除（如果存在）
        boolean removedFromBlockedDeque = blockedDeque.remove(cacheKey);

        if (removedFromBlockedDeque || removedFromDelayedQueue) {
            logger.info("订单 {} 已成功从队列中移除。", cacheKey);
        }
    }

    /**
     * 修改缓存订单状态
     */
    public boolean updateOrderStatusToCache(String cacheKey, Integer status) {
        OrderVo orderVo = selectOrderVoToCache(cacheKey);
        if (orderVo == null) {
            return false;
        }
        if (isOrderInQueues(cacheKey)){
            removeOrderToQueue(cacheKey);
        }
        orderVo.setOrderStatus(status);

        delOrderToCache(orderVo);
        addOrderToCache(orderVo);

        return true;
    }

    /**
     * 向缓存中删除订单vo
     */
    public void delOrderToCache(OrderVo orderVo) {
        String cacheKey = getCacheKey(orderVo.getUserId(), orderVo.getId());
        redisCache.deleteObject(cacheKey);
    }

    /**
     * 清空订单vo缓存数据
     */
    public void clearOrdersCache() {
        Collection<String> keys = redisCache.keys(CacheConstants.VRONE_ORDER_KEY + "*");
        redisCache.deleteObject(keys);
    }

}
