package com.ruoyi.project.service.impl;

import com.ruoyi.project.domain.Notice;
import com.ruoyi.project.domain.Template;
import com.ruoyi.project.domain.Transactions;
import com.ruoyi.project.domain.VrUser;
import com.ruoyi.project.domain.vo.RechargeVo;
import com.ruoyi.project.mapper.NoticeMapper;
import com.ruoyi.project.mapper.TemplateMapper;
import com.ruoyi.project.mapper.TransactionsMapper;
import com.ruoyi.project.mapper.VrUserMapper;
import com.ruoyi.project.service.IDataPanelService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

@Service("dataPanelServiceImplement")
public class DataPanelServiceImpl implements IDataPanelService {

    private final VrUserMapper vrUserMapper;

    private final TransactionsMapper transactionsMapper;

    private final NoticeMapper noticeMapper;

    private final TemplateMapper templateMapper;

    public DataPanelServiceImpl(
            @Autowired VrUserMapper vrUserMapper,
            @Autowired TransactionsMapper transactionsMapper,
            @Autowired NoticeMapper noticeMapper,
            @Autowired TemplateMapper templateMapper) {
        this.vrUserMapper = vrUserMapper;
        this.transactionsMapper = transactionsMapper;
        this.noticeMapper = noticeMapper;
        this.templateMapper = templateMapper;
    }

    /**
     * 获取通知列表
     */
    @Override
    public List<Notice> selectNoticeList() {
        return noticeMapper.selectNoticeListOrderByTime();
    }

    /**
     * 获取模板列表
     */
    @Override
    public List<Template> selectTemplateList() {
        return templateMapper.selectTemplateListOrderByView();
    }

    public String getTimeByDate() {
        Date time = new Date();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");//这里可以指定日期的格式
        return sdf.format(time);
    }

    /**
     * 获取用户充值排行
     */
    @Override
    public List<VrUser> selectUserOrderByMoney() {
        return vrUserMapper.selectUserOrderByMoney();
    }

    /**
     * 查询今日新增用户
     */
    @Override
    public List<VrUser> selectUserByToday() {
        return vrUserMapper.selectUserByToday(getTimeByDate());
    }

    /**
     * 统计今日充值额
     */
    @Override
    public BigDecimal selectTodayMoney() {
        List<Transactions> todayTransactionsList = transactionsMapper.selectTransactionsListByToday(getTimeByDate());
        return todayTransactionsList.stream().map(Transactions::getRecharge).reduce(BigDecimal.ZERO, BigDecimal::add);
    }

    /**
     * 统计周充值额
     */
    @Override
    public List<RechargeVo> selectWeekTransactionsList() {
        List<Transactions> weekTransactionsList = transactionsMapper.selectTransactionsListByWeek(getTimeByDate());
        return getTotalMoneyByDay(weekTransactionsList);
    }

    /**
     * 统计月充值额
     */
    @Override
    public List<RechargeVo> selectMonthTransactionsList() {
        List<Transactions> monthTransactionsList = transactionsMapper.selectTransactionsListByMonth(getTimeByDate());
        return getTotalMoneyByDay(monthTransactionsList);
//        return getTotalMoneyByMonth(rechargeVoList);
    }

    /**
     * 统计一天的数据
     */
    public List<RechargeVo> getTotalMoneyByDay(List<Transactions> list) {
        HashMap<String, RechargeVo> map = new HashMap<>();
        for (Transactions transactions : list) {
            Date date = transactions.getCreateTime();
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            map = getMap(map, date, sdf, transactions.getRecharge());
        }

        return orderMapByTime(map);
    }

    /**
     * 统计一个月的数据
     */
    public List<RechargeVo> getTotalMoneyByMonth(List<RechargeVo> list) {
        HashMap<String, RechargeVo> map = new HashMap<>();
        for (RechargeVo r : list) {
            Date date = r.getTime();
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM");
            map = getMap(map, date, sdf, r.getMoney());
        }

        return orderMapByTime(map);
    }

    /**
     * 累加数据方法
     */
    public HashMap<String, RechargeVo> getMap(HashMap<String, RechargeVo> map, Date date, SimpleDateFormat sdf,
                                              BigDecimal money) {
        String dateStr = sdf.format(date);

        RechargeVo rechargeVo = new RechargeVo();
        if (!map.containsKey(dateStr)) {
            rechargeVo.setMoney(money);
            Date time;
            try {
                time = sdf.parse(dateStr);
            } catch (ParseException e) {
                throw new RuntimeException(e);
            }
            rechargeVo.setTime(time);
        } else {
            rechargeVo = map.get(dateStr);
            rechargeVo.setMoney(rechargeVo.getMoney().add(money));
        }
        map.put(dateStr, rechargeVo);
        return map;
    }

    /**
     * 根据时间排序map
     */
    public List<RechargeVo> orderMapByTime(HashMap<String, RechargeVo> map) {
        List<RechargeVo> rechargeVoList = new ArrayList<>(map.values());
        rechargeVoList.sort(Comparator.comparing(RechargeVo::getTime).reversed());
        return rechargeVoList;
    }

}
