package com.ruoyi.project.service.impl;

import java.util.ArrayList;
import java.util.List;

import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.project.domain.DataSource;
import com.ruoyi.project.domain.sourceData.*;
import com.ruoyi.project.domain.vo.DataSourceVo;
import com.ruoyi.project.mapper.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.project.service.IDataSourceService;

/**
 * 数据源Service业务层处理
 *
 * @author seeseele
 * @date 2024-05-09
 */
@Service("dataSourceServiceImplement")
public class DataSourceServiceImpl implements IDataSourceService {

    private final DataSourceMapper dataSourceMapper;

    private final ImageDataMapper imageDataMapper;

    private final VideoDataMapper videoDataMapper;

    private final LinkDataMapper linkDataMapper;

    private final AlbumDataMapper albumDataMapper;

    private final ModelDataMapper modelDataMapper;

    public DataSourceServiceImpl(@Autowired DataSourceMapper dataSourceMapper,
                                 @Autowired ImageDataMapper imageDataMapper,
                                 @Autowired VideoDataMapper videoDataMapper,
                                 @Autowired LinkDataMapper linkDataMapper,
                                 @Autowired AlbumDataMapper albumDataMapper,
                                 @Autowired ModelDataMapper modelDataMapper) {
        this.dataSourceMapper = dataSourceMapper;
        this.imageDataMapper = imageDataMapper;
        this.videoDataMapper = videoDataMapper;
        this.linkDataMapper = linkDataMapper;
        this.albumDataMapper = albumDataMapper;
        this.modelDataMapper = modelDataMapper;
    }

    /**
     * 将dataSource转化为dataSourceVo
     */
    public DataSourceVo translateToVo(DataSource dataSource) {
        Long dataSourceId = dataSource.getId();
        DataSourceVo dataSourceVo = new DataSourceVo(dataSource);
        if (dataSource.getType() == 1) {
            ImageData imageData = imageDataMapper.selectImageDataByDataSourceId(dataSourceId);
            dataSourceVo.setUrl(imageData.getUrl());
            dataSourceVo.setCover(imageData.getCover());
        } else if (dataSource.getType() == 2) {
            VideoData videoData = videoDataMapper.selectVideoDataByDataSourceId(dataSourceId);
            dataSourceVo.setUrl(videoData.getUrl());
            dataSourceVo.setCover(videoData.getCover());
        }
        return dataSourceVo;
    }

    /**
     * 将dataSourceList转化为dataSourceVoList
     */
    public List<DataSourceVo> translateToVoList(List<DataSource> dataSources) {
        List<DataSourceVo> list = new ArrayList<>();
        for (DataSource d : dataSources) {
            list.add(translateToVo(d));
        }
        return list;
    }

    /**
     * 查询数据源列表
     *
     * @param dataSource 数据源
     * @return 数据源vo
     */
    @Override
    public List<DataSourceVo> selectDataSourceVoList(DataSource dataSource) {
        List<DataSource> dataSources = dataSourceMapper.selectDataSourceList(dataSource);

        return translateToVoList(dataSources);
    }

    /**
     * 查询数据源列表
     *
     * @param appId 展厅id
     * @return 数据源vo
     */
    @Override
    public List<DataSourceVo> selectDataSourceVoListByAppId(Long appId) {
        List<DataSource> dataSources = dataSourceMapper.selectDataSourceListByAppId(appId);
        return translateToVoList(dataSources);
    }

    /**
     * 查询数据源
     *
     * @param id 数据源主键
     * @return 数据源vo
     */
    @Override
    public DataSourceVo selectDataSourceVoById(Long id) {
        DataSource dataSource = dataSourceMapper.selectDataSourceById(id);

        return translateToVo(dataSource);
    }

    /**
     * 新增图片类型的数据源
     *
     * @param dataSource 数据源
     * @param url        图片url
     * @param cover      图片缩略图url
     * @return 结果
     */
    @Override
    public boolean insertDataSourceByImageData(DataSource dataSource, String url, String cover) {
        if (dataSourceMapper.insertDataSource(dataSource) < 1) {
            return false;
        }
        Long dataSourceId = dataSource.getId();

        ImageData imageData = new ImageData();
        imageData.setDataSourceId(dataSourceId);
        imageData.setStorageId(0L);
        imageData.setUrl(url);
        imageData.setCover(cover);
        imageData.setCreateTime(DateUtils.getNowDate());

        return imageDataMapper.insertImageData(imageData) >= 1;
    }

    /**
     * 新增视频类型的数据源
     *
     * @param dataSource 数据源
     * @param url        视频url
     * @param cover      视频封面
     * @return 结果
     */
    @Override
    public boolean insertDataSourceByVideoData(DataSource dataSource, String url, String cover) {
        if (dataSourceMapper.insertDataSource(dataSource) < 1) {
            return false;
        }
        Long dataSourceId = dataSource.getId();

        VideoData videoData = new VideoData();
        videoData.setDataSourceId(dataSourceId);
        videoData.setStorageId(0L);
        videoData.setUrl(url);
        videoData.setCover(cover);
        videoData.setCreateTime(DateUtils.getNowDate());

        return videoDataMapper.insertVideoData(videoData) >= 1;
    }

    /**
     * 新增超链接类型的数据源
     *
     * @param dataSource 数据源
     * @param data        data
     * @param cover      封面
     * @return 结果
     */
    @Override
    public boolean insertDataSourceByLinkData(DataSource dataSource, String data, String cover) {
        if (dataSourceMapper.insertDataSource(dataSource) < 1) {
            return false;
        }
        Long dataSourceId = dataSource.getId();

        LinkData linkData = new LinkData();
        linkData.setDataSourceId(dataSourceId);
        linkData.setStorageId(0L);
        linkData.setData(data);
        linkData.setCover(cover);
        linkData.setCreateTime(DateUtils.getNowDate());

        return linkDataMapper.insertLinkData(linkData) >= 1;
    }

    /**
     * 新增图册类型的数据源
     *
     * @param dataSource 数据源
     * @param data        data
     * @param cover      封面
     * @return 结果
     */
    @Override
    public boolean insertDataSourceByAlbumData(DataSource dataSource, String data, String cover) {
        if (dataSourceMapper.insertDataSource(dataSource) < 1) {
            return false;
        }
        Long dataSourceId = dataSource.getId();

        AlbumData albumData = new AlbumData();
        albumData.setDataSourceId(dataSourceId);
        albumData.setStorageId(0L);
        albumData.setData(data);
        albumData.setCover(cover);
        albumData.setCreateTime(DateUtils.getNowDate());

        return albumDataMapper.insertAlbumData(albumData) >= 1;
    }

    /**
     * 新增模型类型的数据源
     *
     * @param dataSource 数据源
     * @param data        data
     * @param cover      封面
     * @return 结果
     */
    @Override
    public boolean insertDataSourceByModelData(DataSource dataSource, String data, String cover) {
        if (dataSourceMapper.insertDataSource(dataSource) < 1) {
            return false;
        }
        Long dataSourceId = dataSource.getId();

        ModelData modelData = new ModelData();
        modelData.setDataSourceId(dataSourceId);
        modelData.setStorageId(0L);
        modelData.setData(data);
        modelData.setCover(cover);
        modelData.setCreateTime(DateUtils.getNowDate());

        return modelDataMapper.insertModelData(modelData) >= 1;
    }

    /**
     * 修改数据源
     *
     * @param dataSource 数据源
     * @return 结果
     */
    @Override
    public boolean updateDataSource(DataSource dataSource) {
        return dataSourceMapper.updateDataSource(dataSource) > 0;
    }

    /**
     * 修改图片类型的数据源
     *
     * @param dataSource 数据源
     * @param data        图片data
     * @param cover      图片缩略图url
     * @return 结果
     */
    @Override
    public boolean updateDataSourceByImageData(DataSource dataSource, String data, String cover) {
        if (dataSourceMapper.updateDataSource(dataSource) < 1) {
            return false;
        }
        Long dataSourceId = dataSource.getId();

        ImageData imageData = imageDataMapper.selectImageDataByDataSourceId(dataSourceId);
        if (!StringUtils.isNull(data)){
            imageData.setUrl(data);
        }
        if (!StringUtils.isNull(cover)){
            imageData.setCover(cover);
        }
        imageData.setUpdateTime(DateUtils.getNowDate());

        return imageDataMapper.updateImageData(imageData) >= 1;
    }

    /**
     * 修改视频类型的数据源
     *
     * @param dataSource 数据源
     * @param data        视频data
     * @param cover      视频封面
     * @return 结果
     */
    @Override
    public boolean updateDataSourceByVideoData(DataSource dataSource, String data, String cover) {
        if (dataSourceMapper.updateDataSource(dataSource) < 1) {
            return false;
        }
        Long dataSourceId = dataSource.getId();

        VideoData videoData = videoDataMapper.selectVideoDataByDataSourceId(dataSourceId);
        if (!StringUtils.isNull(data)){
            videoData.setUrl(data);
        }
        if (!StringUtils.isNull(cover)){
            videoData.setCover(cover);
        }
        videoData.setUpdateTime(DateUtils.getNowDate());

        return videoDataMapper.updateVideoData(videoData) >= 1;
    }

    /**
     * 修改超链接类型的数据源
     *
     * @param dataSource 数据源
     * @param data        data
     * @param cover      封面
     * @return 结果
     */
    @Override
    public boolean updateDataSourceByLinkData(DataSource dataSource, String data, String cover) {
        if (dataSourceMapper.updateDataSource(dataSource) < 1) {
            return false;
        }
        Long dataSourceId = dataSource.getId();

        LinkData linkData = linkDataMapper.selectLinkDataByDataSourceId(dataSourceId);
        if (!StringUtils.isNull(data)){
            linkData.setData(data);
        }
        if (!StringUtils.isNull(cover)){
            linkData.setCover(cover);
        }
        linkData.setUpdateTime(DateUtils.getNowDate());

        return linkDataMapper.updateLinkData(linkData) >= 1;
    }

    /**
     * 修改图册类型的数据源
     *
     * @param dataSource 数据源
     * @param data        data
     * @param cover      封面
     * @return 结果
     */
    @Override
    public boolean updateDataSourceByAlbumData(DataSource dataSource, String data, String cover) {
        if (dataSourceMapper.updateDataSource(dataSource) < 1) {
            return false;
        }
        Long dataSourceId = dataSource.getId();

        AlbumData albumData = albumDataMapper.selectAlbumDataByDataSourceId(dataSourceId);
        if (!StringUtils.isNull(data)){
            albumData.setData(data);
        }
        if (!StringUtils.isNull(cover)){
            albumData.setCover(cover);
        }
        albumData.setUpdateTime(DateUtils.getNowDate());

        return albumDataMapper.updateAlbumData(albumData) >= 1;
    }

    /**
     * 修改模型类型的数据源
     *
     * @param dataSource 数据源
     * @param data        data
     * @param cover      封面
     * @return 结果
     */
    @Override
    public boolean updateDataSourceByModelData(DataSource dataSource, String data, String cover) {
        if (dataSourceMapper.updateDataSource(dataSource) < 1) {
            return false;
        }
        Long dataSourceId = dataSource.getId();

        ModelData modelData = new ModelData();
        modelData.setDataSourceId(dataSourceId);
        modelData.setStorageId(0L);
        modelData.setData(data);
        modelData.setCover(cover);
        modelData.setUpdateTime(DateUtils.getNowDate());

        return modelDataMapper.updateModelData(modelData) >= 1;
    }

    /**
     * 批量删除数据源
     *
     * @param ids 需要删除的数据源主键
     * @return 结果
     */
    @Override
    public boolean deleteDataSourceByIds(Long[] ids) {
        return dataSourceMapper.deleteDataSourceByIds(ids) > 0;
    }

    /**
     * 删除数据源信息
     *
     * @param id 数据源主键
     * @return 结果
     */
    @Override
    public boolean deleteDataSourceById(Long id) {
        return dataSourceMapper.deleteDataSourceById(id) > 0;
    }

}
