package com.ruoyi.project.service.impl;

import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.util.IdUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.common.BulkImportService;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.project.domain.*;
import com.ruoyi.project.domain.vo.OrderVo;
import com.ruoyi.project.mapper.*;
import com.ruoyi.project.service.IOrdersService;
import com.ruoyi.project.service.cache.OrderCacheService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.PostConstruct;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 订单Service业务层处理
 *
 * @author seeseele
 * @date 2025-02-26
 */
@Service("ordersServiceImplement")
public class OrdersServiceImpl implements IOrdersService {

    private final OrdersMapper ordersMapper;

    private final OrderDetailsMapper orderDetailsMapper;

    private final VrUserMapper vrUserMapper;

    private final TransactionsMapper transactionsMapper;

    private final TemplateMapper templateMapper;

    private final TemplateLogMapper templateLogMapper;

    private final SceneMapper sceneMapper;

    private final OrderCacheService orderCacheService;

    public OrdersServiceImpl(
            @Autowired OrdersMapper ordersMapper,
            @Autowired OrderDetailsMapper orderDetailsMapper,
            @Autowired VrUserMapper vrUserMapper,
            @Autowired TransactionsMapper transactionsMapper,
            @Autowired TemplateMapper templateMapper,
            @Autowired TemplateLogMapper templateLogMapper,
            @Autowired SceneMapper sceneMapper,
            @Autowired OrderCacheService orderCacheService) {
        this.ordersMapper = ordersMapper;
        this.orderDetailsMapper = orderDetailsMapper;
        this.vrUserMapper = vrUserMapper;
        this.transactionsMapper = transactionsMapper;
        this.templateMapper = templateMapper;
        this.templateLogMapper = templateLogMapper;
        this.sceneMapper = sceneMapper;
        this.orderCacheService = orderCacheService;
    }

    /**
     * 项目启动时，初始化订单vo到缓存
     */
    @PostConstruct
    public void init() {
        loadingOrdersCache();
    }

    /**
     * 加载订单vo缓存数据
     */
    public void loadingOrdersCache() {
        List<Orders> ordersList = ordersMapper.selectOrdersList(new Orders());
        for (Orders orders : ordersList) {
            orderCacheService.addOrderToCache(transformToVo(orders));
        }
    }

    /**
     * 重置订单vo缓存数据
     */
    public void resetOrdersCache() {
        orderCacheService.clearOrdersCache();
        loadingOrdersCache();
    }

    /**
     * 将Orders对象转换为OrderVo对象
     */
    public OrderVo transformToVo(Orders orders) {
        List<OrderDetails> orderDetailsList = orderDetailsMapper.selectOrderDetailsByOrderId(orders.getId());

        return new OrderVo(orders, orderDetailsList);
    }

    /**
     * 通过用户id获取OrderVo列表
     */
    @Override
    public List<OrderVo> selectOrderVoListByUserId(Long userId) {
        Map<String, Object> namespaceData = orderCacheService.getNamespaceData();
        List<OrderVo> orderVoList = new ArrayList<>();
        for (String key : namespaceData.keySet()) {
            // vrone:1&012934 通过键获取userId
            String[] keys = key.split(":")[1].split("&");
            if (Long.parseLong(keys[0]) == userId) {
                OrderVo orderVo = orderCacheService.selectOrderVoToCache(key);
                orderVoList.add(orderVo);
            }
        }

        return orderVoList;
    }

    /**
     * 根据模板id查询模板
     */
    @Override
    public Template selectTemplateById(Long templateId) {
        return templateMapper.selectTemplateById(templateId);
    }

    /**
     * 查询订单
     *
     * @return 订单
     */
    @Override
    public OrderVo selectOrdersToCache(Long userId, Long orderId) {
        return orderCacheService.selectOrderVoToCache(userId, orderId);
    }

    /**
     * 创建订单
     */
    @Override
    public OrderVo createOrderVo(Orders orders, List<OrderDetails> orderDetailsList) {
        Snowflake orderSnowflake = IdUtil.getSnowflake(1, 1);  // 用于生成订单 ID
        Snowflake detailSnowflake = IdUtil.getSnowflake(2, 2); // 用于生成订单详情 ID，确保与订单 ID 不冲突

        // 生成雪花 ID
        long orderId = orderSnowflake.nextId();
        String shortOrderId = String.valueOf(orderId).substring(0, 16); // 截取前16位

        // 为订单id生成雪花算法
        orders.setId(Long.parseLong(shortOrderId));
        //订单详情id生成雪花算法
        for (OrderDetails orderDetails : orderDetailsList) {

            long detailId = detailSnowflake.nextId();
            String shortDetailId = String.valueOf(detailId).substring(0, 16);

            orderDetails.setId(Long.parseLong(shortDetailId));
            orderDetails.setOrderId(Long.parseLong(shortOrderId));
        }

        // 创建订单vo，并将其添加到队列和缓存中
        OrderVo orderVo = new OrderVo(orders, orderDetailsList);
        orderCacheService.insertOrderToQueue(orderVo);
        orderCacheService.addOrderToCache(orderVo);

        return orderVo;
    }

    /**
     * 支付订单
     */
    @Override
    @Transactional
    public AjaxResult payment(Long userId, Long orderId) {
        // 获取订单详情
        OrderVo orderVo = orderCacheService.selectOrderVoToCache(userId, orderId);
        // 获取用户
        VrUser vrUser = vrUserMapper.selectVrUserById(userId);
        // 判断用户余额是否充足
        BigDecimal balance = vrUser.getMoney().subtract(orderVo.getTotalAmount());
        if (vrUser.getMoney().compareTo(orderVo.getTotalAmount()) < 0) {
            return AjaxResult.error("余额不足");
        }

        // 创建支付记录
        Long transactionsId = paymentLogControls(userId, orderId, orderVo.getTotalAmount(), balance);
        if (transactionsId.equals(0L)) {
            return AjaxResult.error("支付失败");
        }

        // 用户扣款
        if (!feeDeductionControls(vrUser, balance, transactionsId)) {
            return AjaxResult.error("扣款失败");
        }

        // 添加模板与用户相关记录
        int successNum = 0;    // 成功消息数
        int failureNum = 0;    // 失败消息数
        // 失败消息详情
        final StringBuilder failureMsg = new StringBuilder();
        List<OrderDetails> orderDetailsList = orderVo.getOrderDetailsList();
        for (OrderDetails orderDetails : orderDetailsList) {
            // 添加记录
            if (!addTemplateLogControls(userId, orderDetails)) {
                // 返还未购买成功的模板金额
                String result = returnTemplateAmount(orderDetails, vrUser);
                failureNum++;
                failureMsg.append(result).append("\n");
                continue;
            }
            successNum++;
        }

        // 修改订单状态
        orderCacheService.updateOrderStatusToCache(orderCacheService.getCacheKey(userId, orderId), 2);

        if (successNum == 0) {
            return AjaxResult.error("支付失败", failureMsg.toString());
        }
        if (failureNum > 0) {
            return AjaxResult.error("部分订单支付失败", failureMsg.toString());
        }
        return AjaxResult.success("支付成功");
    }

    /**
     * 创建支付记录相关操作
     */
    public Long paymentLogControls(Long userId,Long orderId, BigDecimal recharge, BigDecimal balance) {
        // 添加支付记录
        Transactions transactions = new Transactions();
        transactions.setUserId(userId);
        transactions.setRecharge(recharge);
        transactions.setMoney(balance);
        transactions.setPaymentMethod(4);
        transactions.setPaymentType(1);
        transactions.setStatus(1);
        transactions.setOrderId(orderId);
        if (transactionsMapper.insertTransactions(transactions) < 1) {
            return 0L;
        }

        return transactions.getId();
    }

    /**
     * 用户扣款相关操作
     */
    public boolean feeDeductionControls(VrUser vrUser, BigDecimal balance, Long transactionId) {
        // 更新用户余额
        vrUser.setMoney(balance);
        vrUser.setUpdateTime(DateUtils.getNowDate());
        Transactions transactions = transactionsMapper.selectTransactionsById(transactionId);

        if (vrUserMapper.updateVrUser(vrUser) < 1) {
            transactions.setStatus(0);
        }else {
            transactions.setStatus(2);
        }

        return transactionsMapper.updateTransactions(transactions) > 0;
    }

    /**
     * 添加模板与用户相关记录相关操作
     */
    public boolean addTemplateLogControls(Long userId, OrderDetails orderDetails) {
        TemplateLog templateLog = new TemplateLog();
        templateLog.setTemplateId(orderDetails.getTemplateId());
        templateLog.setUserId(userId);
        templateLog.setCreateTime(DateUtils.getNowDate());
        return templateLogMapper.insertTemplateLog(templateLog) > 0;
    }

    /**
     * 返还模板金额
     */
    public String returnTemplateAmount(OrderDetails orderDetails, VrUser vrUser) {
        String baseStr = "模板id:" + orderDetails.getTemplateId();
        // 返还未购买成功的模板金额
        BigDecimal balance = vrUser.getMoney().add(orderDetails.getPrice());
        vrUser.setMoney(balance);
        vrUser.setUpdateTime(DateUtils.getNowDate());
        if (vrUserMapper.updateVrUser(vrUser) < 1) {
            return "返还用户余额失败;" + baseStr;
        }
        // 添加支付记录
        Transactions t = new Transactions();
        t.setUserId(vrUser.getId());
        t.setOrderId(orderDetails.getOrderId());
        t.setRecharge(orderDetails.getPrice());
        t.setMoney(balance);
        t.setPaymentMethod(4);
        t.setPaymentType(2);
        t.setStatus(2);
        t.setMark("添加模板记录失败,返还金额");
        if (transactionsMapper.insertTransactions(t) < 1) {
            return "返回余额成功，创建支付记录失败;" + baseStr;
        }
        return "返还模板金额成功" + baseStr;
    }

    /**
     * 取消订单
     */
    @Override
    public AjaxResult cancelOrder(Long userId, Long orderId) {
        String cacheKey = orderCacheService.getCacheKey(userId, orderId);
        orderCacheService.removeOrderToQueue(cacheKey);
        boolean result = orderCacheService.updateOrderStatusToCache(cacheKey, 3);

        if (result) {
            return AjaxResult.success("订单取消成功");
        }
        return AjaxResult.error("订单取消失败");
    }

    /**
     * 订单退款
     */
    @Override
    @Transactional
    public AjaxResult refundOrder(Long userId, Long orderId) {
        // 获取订单vo
        OrderVo orderVo = orderCacheService.selectOrderVoToCache(userId, orderId);
        if (orderVo == null) {
            return AjaxResult.error("订单不存在");
        }
        // 获取用户
        VrUser vrUser = vrUserMapper.selectVrUserById(userId);

        // 记录退款记录
        // 消息详情
        final StringBuilder msg = new StringBuilder();
        List<OrderDetails> orderDetailsList = orderVo.getOrderDetailsList();
        for (OrderDetails orderDetails : orderDetailsList) {
            if (orderDetails.getStatus() == 1) {
                continue;
            }

            // 判断模板是否使用
            List<Scene> sceneList = sceneMapper.selectSceneListByTemplateId(orderDetails.getTemplateId());
            if (!sceneList.isEmpty()) {
                msg
                        .append("模板名id：")
                        .append(orderDetails.getTemplateId())
                        .append("模板已被使用，无法退款;")
                        .append("\n");
            } else {
                // 返还未使用的模板金额
                String result = returnTemplateAmount(orderDetails, vrUser);
                msg.append(result).append("\n");
            }
        }

        // 更新订单状态
        orderCacheService.updateOrderStatusToCache(orderCacheService.getCacheKey(userId, orderId), 3);

        return AjaxResult.success(msg);
    }

    /**
     * 将缓存中的数据转移到数据库
     */
    @Override
    public void persistRedisCacheToDb() {
        Map<String, Object> namespaceData = orderCacheService.getNamespaceData();
        List<Orders> ordersList = new ArrayList<>();
        List<OrderDetails> orderDetailsList = new ArrayList<>();
        for (String key : namespaceData.keySet()) {
            OrderVo orderVo = getOrderVoFromNamespaceData(namespaceData, key);
            if (orderVo == null) {
                continue;
            }

            Orders orders = new Orders();
            orders.setId(orderVo.getId());
            orders.setUserId(orderVo.getUserId());
            orders.setTotalAmount(orderVo.getTotalAmount());
            orders.setOrderDate(orderVo.getOrderDate());
            orders.setOrderStatus(orderVo.getOrderStatus());
            orders.setRefundStatus(orderVo.getRefundStatus());
            orders.setRefundDate(orderVo.getRefundDate());

            ordersList.add(orders);
            orderDetailsList.addAll(orderVo.getOrderDetailsList());
        }

        String ordersData = importOrdersData(ordersList, true);
        String orderDetailsData = importOrderDetailsData(orderDetailsList, true);
        System.out.println("将缓存中的数据转移到数据库成功！");
        System.out.println("订单数据：" + ordersData);
        System.out.println("订单详情数据：" + orderDetailsData);
        resetOrdersCache();
    }

    /**
     * 从 namespaceData 中获取 OrderVo 对象
     */
    private OrderVo getOrderVoFromNamespaceData(Map<String, Object> namespaceData, String key) {
        Object value = namespaceData.get(key);
        if (value == null) {
            return null;
        }
        if (value instanceof JSONObject) {
            JSONObject jsonObject = (JSONObject) value;

            return JSON.parseObject(jsonObject.toJSONString(), OrderVo.class);
        } else {
            throw new ClassCastException("期望 JSONObject 类型，但找到 " + value.getClass().getName());
        }
    }

    /**
     * 导入Order数据
     *
     * @param dataList 数据列表
     * @return 结果
     */
    public String importOrdersData(List<Orders> dataList, boolean updateSupport) {
        BulkImportService<Orders> importService = new BulkImportService<Orders>() {
            @Override
            protected Long checkExistingItem(Orders item) {
                Orders orders = ordersMapper.selectOrdersById(item.getId());
                return orders == null ? 0L : orders.getId();
            }

            @Override
            protected boolean updateItem(Orders item, Long id) {
                item.setId(id);
                return ordersMapper.updateOrders(item) > 0;
            }

            @Override
            protected boolean executeBatch(List<Orders> batchDataList) {
                return ordersMapper.insertOrdersDataList(batchDataList) > 0;
            }

            @Override
            protected String getItemTitle(Orders item) {
                return item.getId().toString();
            }
        };

        return importService.importData(dataList, updateSupport);
    }

    /**
     * 导入OrderDetails数据
     *
     * @param dataList 数据列表
     * @return 结果
     */
    public String importOrderDetailsData(List<OrderDetails> dataList, boolean updateSupport) {
        BulkImportService<OrderDetails> importService = new BulkImportService<OrderDetails>() {
            @Override
            protected Long checkExistingItem(OrderDetails item) {
                OrderDetails orderDetails = orderDetailsMapper.selectOrderDetailsById(item.getId());
                return orderDetails == null ? 0L : orderDetails.getId();
            }

            @Override
            protected boolean updateItem(OrderDetails item, Long id) {
                item.setId(id);
                return orderDetailsMapper.updateOrderDetails(item) > 0;
            }

            @Override
            protected boolean executeBatch(List<OrderDetails> batchDataList) {
                return orderDetailsMapper.insertOrderDetailsDataList(batchDataList) > 0;
            }

            @Override
            protected String getItemTitle(OrderDetails item) {
                return item.getId().toString();
            }
        };

        return importService.importData(dataList, updateSupport);
    }
}
