package com.ruoyi.project.service.impl;


import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.utils.CommonUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.framework.config.ServerConfig;
import com.ruoyi.project.domain.DataSource;
import com.ruoyi.project.domain.Scene;
import com.ruoyi.project.domain.Spot;
import com.ruoyi.project.domain.vo.DataSourceVo;
import com.ruoyi.project.domain.vo.SpotVo;
import com.ruoyi.project.mapper.DataSourceMapper;
import com.ruoyi.project.mapper.ImageDataMapper;
import com.ruoyi.project.mapper.SceneMapper;
import com.ruoyi.project.mapper.VideoDataMapper;
import com.ruoyi.project.service.ISpotService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

@Service("spotServiceImplement")
public class SpotServiceImpl implements ISpotService {

    private final SceneMapper sceneMapper;

    private final DataSourceMapper dataSourceMapper;

    private final ImageDataMapper imageDataMapper;

    private final VideoDataMapper videoDataMapper;

    private final ServerConfig serverConfig;

    public SpotServiceImpl(
            @Autowired SceneMapper sceneMapper,
            @Autowired DataSourceMapper dataSourceMapper,
            @Autowired ImageDataMapper imageDataMapper,
            @Autowired VideoDataMapper videoDataMapper,
            @Autowired ServerConfig serverConfig) {
        this.sceneMapper = sceneMapper;
        this.dataSourceMapper = dataSourceMapper;
        this.imageDataMapper = imageDataMapper;
        this.videoDataMapper = videoDataMapper;
        this.serverConfig = serverConfig;
    }

    public String getSpotUrl(String spotPath) {
        return serverConfig.getUrl() + spotPath;
    }

    /**
     * 根据场景id查询展示点列表
     *
     * @param sceneId 场景id
     */
    @Override
    public List<SpotVo> selectSpotListBySceneId(Long sceneId) {
        Scene scene = sceneMapper.selectSceneById(sceneId);
        // 根据储存的spot文件查询spot
        String spotStr = CommonUtils.getContentByUrl(getSpotUrl(scene.getSpot()), String.class);

        List<Spot> spotList = CommonUtils.translateToList(spotStr, "spots", Spot.class);

        HashMap<String, DataSourceVo> dataMap = CommonUtils.getDataSourceVoMap(scene, dataSourceMapper, imageDataMapper, videoDataMapper);

        List<SpotVo> spotVoList = new ArrayList<>();
        for (Spot s : spotList) {
            SpotVo spotVo = new SpotVo();
            spotVo.setId(s.getId());
            spotVo.setName(s.getName());
            spotVo.setType(s.getType());
            spotVo.setConfig(s.getConfig());

            String spotTitle = sceneId + "&" + s.getId();
            if (dataMap.containsKey(spotTitle)) {
                DataSourceVo d = dataMap.get(spotTitle);
                spotVo.setDataSourceId(d.getId());
                spotVo.setDataUrl(d.getUrl());
                spotVo.setCoverUrl(d.getCover());
            }
            spotVoList.add(spotVo);
        }
        return spotVoList;
    }

    /**
     * 根据展示点标题查询数据源
     */
    @Override
    public DataSource selectDataSourceBySpot(String spotTitle) {
        return dataSourceMapper.selectDataSourceBySpot(spotTitle);
    }

    /**
     * 根据场景id修改展示点
     *
     * @param sceneId id
     * @param spot    展示点
     */
    @Override
    public boolean updateSpotBySceneId(Long sceneId, Spot spot) {
        Scene scene = sceneMapper.selectSceneById(sceneId);
        // 根据储存的spot文件查询spot
        JSONObject spotJson = CommonUtils.getContentByUrl(getSpotUrl(scene.getSpot()), JSONObject.class);
        List<Spot> spotList = CommonUtils.translateToList(spotJson.toJSONString(), "spots", Spot.class);
        // 遍历spotList，找到要修改的spot，然后替换掉
        for (Spot s : spotList) {
            if (s.getId().equals(spot.getId())) {
                spotList.remove(s);
                spotList.add(spot);
                break;
            }
        }
        // 删除旧spot文件
        CommonUtils.deleteSpotFile(scene.getSpot());

        // 更新spot文件
        spotJson.put("spots", spotList);
        // 创建新spot文件
        AjaxResult createSpotFile = CommonUtils.createSpotFile(spotJson);
        if (!createSpotFile.isSuccess()) {
            return false;
        }

        // 设置场景的 spot 字段
        scene.setSpot(createSpotFile.get("data").toString());

        return sceneMapper.updateScene(scene) >= 1;
    }

    /**
     * 判断数据源是否已挂载
     */
    String judgeMount(Long dataSourceId) {
        DataSource dataSource = dataSourceMapper.selectDataSourceById(dataSourceId);

        return dataSource.getSpot();
    }

    /**
     * 挂载数据源
     *
     * @param dataSourceId 数据源id
     * @param spotTitle    展示点标题
     */
    @Override
    public boolean mountSpot(Long dataSourceId, String spotTitle) {
        String spot = judgeMount(dataSourceId);
        // 判断数据源是否已挂载
        if (!StringUtils.isEmpty(spot)) {
            spot = spot + "#" + spotTitle;
        } else {
            spot = spotTitle;
        }
        return dataSourceMapper.updateSpot(dataSourceId, spot) > 0;
    }

    /**
     * 卸载数据源
     *
     * @param dataSourceId 数据源id
     * @param spotTitle    展示点标题
     */
    @Override
    public boolean unloadSpot(Long dataSourceId, String spotTitle) {
        String spot = judgeMount(dataSourceId);
        // 判断数据源是否已挂载
        if (!StringUtils.isEmpty(spot)) {
            String[] spotList = spot.split("#");
            StringBuilder newSpot = new StringBuilder();
            for (String s : spotList) {
                if (s.equals(spotTitle)) {
                    continue;
                }
                if (newSpot.length() > 0) {
                    newSpot.append("#");
                }
                newSpot.append(s);
            }
            spot = newSpot.toString();
        }
        return dataSourceMapper.updateSpot(dataSourceId, spot) > 0;
    }

}
