package com.ruoyi.project.service.impl;

import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.PasswordUtils;
import com.ruoyi.project.domain.VrUser;
import com.ruoyi.project.mapper.VrUserMapper;
import com.ruoyi.project.service.IVrUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * vr用户Service业务层处理
 *
 * @author seeseele
 * @date 2025-03-10
 */
@Service("vrUserServiceImplement")
public class VrUserServiceImpl implements IVrUserService {

    private final VrUserMapper vrUserMapper;

    public VrUserServiceImpl(
            @Autowired VrUserMapper vrUserMapper) {
        this.vrUserMapper = vrUserMapper;
    }

    /**
     * 手机号密码注册
     */
    @Override
    public AjaxResult register(VrUser vrUser) {
        // **1. 校验手机号是否为空**
        if (vrUser.getPhone() == null || vrUser.getPhone().trim().isEmpty()) {
            return AjaxResult.error("手机号不能为空");
        }

        // **2. 校验手机号是否为 11 位数字**
        if (!vrUser.getPhone().matches("^\\d{11}$")) {
            return AjaxResult.error("手机号必须是11位数字");
        }

        // **3. 校验用户名和密码是否为空**
        if (vrUser.getUserName() == null || vrUser.getUserName().trim().isEmpty()) {
            return AjaxResult.error("用户名不能为空");
        }
        if (vrUser.getPassword() == null || vrUser.getPassword().trim().isEmpty()) {
            return AjaxResult.error("密码不能为空");
        }

        if (vrUserMapper.selectVrUserByPhone(vrUser.getPhone()) != null) {
            return AjaxResult.error("手机号已被注册");
        }
        if (vrUserMapper.selectVrUserByEmail(vrUser.getEmail()) != null) {
            return AjaxResult.error("邮箱已被注册");
        }
        if (vrUserMapper.selectVrUserByUserName(vrUser.getUserName()) != null) {
            return AjaxResult.error("用户名已被注册");
        }

        // 加密密码
        String encryptedPassword = PasswordUtils.encryptPassword(vrUser.getPassword());
        vrUser.setPassword(encryptedPassword); // 设置加密后的密码

        if (vrUserMapper.insertVrUser(vrUser) < 1) {
            return AjaxResult.error("用户注册失败");
        }

        return AjaxResult.success("用户注册成功");
    }

    /**
     * 手机号密码登录
     */
    @Override
    public VrUser login(String phone, String password) {
        VrUser user = vrUserMapper.selectVrUserByPhone(phone);
        if (user == null) {
            return null;
        } else if (!PasswordUtils.matchPassword(password, user.getPassword())) {
            return null;
        }

        return user;
    }

    /**
     * 查询vr用户
     *
     * @param id vr用户主键
     * @return vr用户
     */
    @Override
    public VrUser selectVrUserById(Long id) {
        return vrUserMapper.selectVrUserById(id);
    }

    /**
     * 查询vr用户列表
     *
     * @param vrUser vr用户
     * @return vr用户
     */
    @Override
    public List<VrUser> selectVrUserList(VrUser vrUser) {
        return vrUserMapper.selectVrUserList(vrUser);
    }

    /**
     * 新增vr用户
     *
     * @param vrUser vr用户
     * @return 结果
     */
    @Override
    public int insertVrUser(VrUser vrUser) {
        vrUser.setCreateTime(DateUtils.getNowDate());
        return vrUserMapper.insertVrUser(vrUser);
    }

    /**
     * 修改vr用户
     *
     * @param vrUser vr用户
     * @return 结果
     */
    @Override
    public int updateVrUser(VrUser vrUser) {
        vrUser.setUpdateTime(DateUtils.getNowDate());
        return vrUserMapper.updateVrUser(vrUser);
    }

    /**
     * 修改vr用户密码
     *
     * @return 结果
     */
    @Override
    public boolean updateVrUserPassword(Long userId, String newPassword) {
        // 加密密码
        String encryptedPassword = PasswordUtils.encryptPassword(newPassword);

        return vrUserMapper.updateVrUserPassword(userId, encryptedPassword) > 0;
    }

    /**
     * 修改vr用户邮箱
     *
     * @return 结果
     */
    @Override
    public boolean updateVrUserEmail(Long userId, String newEmail) {
        return vrUserMapper.updateVrUserEmail(userId, newEmail) > 0;
    }

    /**
     * 修改vr用户头像
     *
     * @return 结果
     */
    @Override
    public boolean updateVrUserAvatar(Long userId, String newAvatar) {
        return vrUserMapper.updateVrUserAvatar(userId, newAvatar) > 0;
    }

    /**
     * 批量删除vr用户
     *
     * @param ids 需要删除的vr用户主键
     * @return 结果
     */
    @Override
    public int deleteVrUserByIds(Long[] ids) {
        return vrUserMapper.deleteVrUserByIds(ids);
    }

    /**
     * 删除vr用户信息
     *
     * @param id vr用户主键
     * @return 结果
     */
    @Override
    public int deleteVrUserById(Long id) {
        return vrUserMapper.deleteVrUserById(id);
    }
}
