package com.ruoyi.scheduler;

import com.ruoyi.common.constant.OrderConst;
import com.ruoyi.project.domain.vo.OrderVo;
import com.ruoyi.project.service.IOrdersService;
import com.ruoyi.project.service.cache.OrderCacheService;
import org.redisson.api.RBlockingDeque;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.DisposableBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

@Component
public class OrderTask implements DisposableBean {

    private final IOrdersService orderService;

    private final OrderCacheService orderCacheService;

    private final RedissonClient redissonClient;
    // 线程池
    private ExecutorService executorService;
    // 线程运行
    private volatile boolean running = true;

    public OrderTask(
            @Qualifier("ordersServiceImplement") IOrdersService orderService,
            @Autowired OrderCacheService orderCacheService,
            @Autowired RedissonClient redissonClient) {
        this.orderService = orderService;
        this.orderCacheService = orderCacheService;
        this.redissonClient = redissonClient;
    }

    /**
     * 初始化时启动队列监听线程
     */
    @PostConstruct
    public void init() {
        executorService = Executors.newSingleThreadExecutor();
        executorService.submit(this::listenDelayedQueue);
    }

    /**
     * 监听延迟队列的线程方法
     * 处理延迟队列中的到期订单
     */
    private void listenDelayedQueue() {
        RBlockingDeque<String> blockingDeque = redissonClient.getBlockingDeque(OrderConst.CANCEL_QUEUE_KEY);
        while (running && !Thread.currentThread().isInterrupted()) {
            try {
                // 阻塞直到有元素到达
                String cacheKey = blockingDeque.take();
                OrderVo orderVo = orderCacheService.selectOrderVoToCache(cacheKey);
                if (orderVo.getOrderStatus() == 0){
                    // 处理到期的订单
                    //指定日期格式
                    DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
                    System.out.println(
                            "订单 " + cacheKey + " 已到期，取消订单; 执行时间：" + formatter.format(LocalDateTime.now()));
                    orderCacheService.updateOrderStatusToCache(cacheKey, 3);
                }
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
                break;
            } catch (Exception e) {
                // 处理其他异常
                System.err.println("处理延迟队列中的订单时发生错误: " + e.getMessage());
            }
        }
    }

    /**
     * 销毁时关闭线程池
     */
    @Override
    public void destroy() {
        running = false;
        if (executorService != null) {
            executorService.shutdown();
            try {
                if (!executorService.awaitTermination(5, TimeUnit.SECONDS)) {
                    executorService.shutdownNow();
                }
            } catch (InterruptedException e) {
                executorService.shutdownNow();
            }
        }

    }

    /**
     * 将redis缓存中的数据转移到数据库
     */
    @Scheduled(cron = "0 0 4 * * ?") // 每天04:00执行
    public void syncRedisToDatabase() {
        orderService.persistRedisCacheToDb();
    }
}
