package com.ruoyi.service.impl;

import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.util.IdUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.common.MBulkImportService;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.domain.*;
import com.ruoyi.domain.vo.MOrderDetailVo;
import com.ruoyi.domain.vo.MOrdersVo;
import com.ruoyi.mapper.*;
import com.ruoyi.service.MOrdersService;
import com.ruoyi.service.MTemplateService;
import com.ruoyi.service.cache.MOrderCacheService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.PostConstruct;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @author xuanxuan
 */
@Service
public class MOrdersServiceImpl implements MOrdersService {

    @Autowired
    private MTemplateMapper templateMapper;

    @Autowired
    private MOrdersMapper ordersMapper;

    @Autowired
    private UserMapper userMapper;

    @Autowired
    private MTransactionsMapper transactionsMapper;

    @Autowired
    private MTemplateLogMapper templateLogMapper;

    @Autowired
    private final Snowflake snowflake;

    @Autowired
    private MOrderDetailsMapper orderDetailsMapper;

    @Autowired
    private MOrderCacheService orderCacheService;

    @Autowired
    private MTemplateService templateService;

    public MOrdersServiceImpl(Snowflake snowflake) {
        this.snowflake = snowflake;
    }

    /**
     * 重置订单vo缓存数据
     */
    public void resetOrdersCache() {
        orderCacheService.clearOrdersCache();
        loadingOrdersCache();
    }

    /**
     * 项目启动时，初始化订单vo到缓存
     */
    @PostConstruct
    public void init() {
        loadingOrdersCache();
    }

    /**
     * 加载订单vo缓存数据
     */
    public void loadingOrdersCache() {
        List<MOrders> ordersList = ordersMapper.selectOrdersList(new MOrders());
        for (MOrders orders : ordersList) {
            orderCacheService.addOrderToCache(transformToVo(orders));
        }
    }

    /**
     * 将Orders对象转换为OrderVo对象
     */
    public MOrdersVo transformToVo(MOrders orders) {
        List<MOrderDetails> orderDetailsList = orderDetailsMapper.selectOrderDetailsByOrderId(orders.getId());

        return new MOrdersVo(orders, orderDetailsList);
    }

    /**
     * 根据模板id查询模板
     */
    @Override
    public MTemplate selectTemplateById(Long templateId) {
        return templateMapper.selectTemplateById(templateId);
    }

    /**
     * 创建订单
     */
    @Override
    public MOrdersVo createOrderVo(MOrders orders, List<MOrderDetails> orderDetailsList) {

        Snowflake orderSnowflake = IdUtil.getSnowflake(1, 1);  // 用于生成订单 ID
        Snowflake detailSnowflake = IdUtil.getSnowflake(2, 2); // 用于生成订单详情 ID，确保与订单 ID 不冲突

        // 生成雪花 ID
        long orderId = orderSnowflake.nextId();
        String shortOrderId = String.valueOf(orderId).substring(0, 16); // 截取前16位

        // 为订单id生成雪花算法
        orders.setId(Long.parseLong(shortOrderId));
        //订单详情id生成雪花算法
        for (MOrderDetails orderDetails : orderDetailsList) {

            long detailId = detailSnowflake.nextId();
            String shortDetailId = String.valueOf(detailId).substring(0, 16);



            orderDetails.setId(Long.parseLong(shortDetailId));
            orderDetails.setOrderId(Long.parseLong(shortOrderId));

            // 🌟 这里补充：根据模板ID查模板信息
            MTemplate template = templateService.selectTemplateById(orderDetails.getTemplateId());
            if (template != null) {
                orderDetails.setTemplateName(template.getTitle());
                orderDetails.setImgUrl(template.getImgUrl());
            }
        }

        // 创建订单vo，并将其添加到队列和缓存中
        MOrdersVo orderVo = new MOrdersVo(orders, orderDetailsList);

        orderCacheService.insertOrderToQueue(orderVo);
        orderCacheService.addOrderToCache(orderVo);

        return orderVo;
    }

    /**
     * 根据用户id查询订单vo列表
     */
    @Override
    public List<MOrdersVo> selectOrderVoListByUserId(long userId) {
        Map<String, Object> namespaceData = orderCacheService.getNamespaceData();
        List<MOrdersVo> orderVoList = new ArrayList<>();
        for (String key : namespaceData.keySet()) {
            // vrone:1&012934 通过键获取userId
            String[] keys = key.split(":")[1].split("&");
            if (Long.parseLong(keys[0]) == userId) {
                MOrdersVo orderVo = orderCacheService.selectOrderVoToCache(key);
                orderVoList.add(orderVo);
            }
        }

        return orderVoList;
    }

    /***
     *
     * @param userId 用户id
     * @param orderId 订单id
     * @return
     */
    @Override
    public MOrdersVo selectOrdersToCache(long userId, Long orderId) {
        return orderCacheService.selectOrderVoToCache(userId, orderId);
    }

    /**
     * 将缓存中的数据转移到数据库
     */
    @Override
    @Transactional
    public void persistRedisCacheToDb() {
        Map<String, Object> namespaceData = orderCacheService.getNamespaceData();
        List<MOrders> ordersList = new ArrayList<>();
        List<MOrderDetails> orderDetailsList = new ArrayList<>();
        for (String key : namespaceData.keySet()) {
            MOrdersVo orderVo = getOrderVoFromNamespaceData(namespaceData, key);
            if (orderVo == null) {
                continue;
            }

            MOrders orders = new MOrders();
            orders.setId(orderVo.getId());
            orders.setUserId(orderVo.getUserId());
            orders.setTotalAmount(orderVo.getTotalAmount());
            orders.setOrderDate(orderVo.getOrderDate());
            orders.setOrderStatus(orderVo.getOrderStatus());
            orders.setRefundStatus(orderVo.getRefundStatus());
            orders.setRefundDate(orderVo.getRefundDate());

            ordersList.add(orders);
            orderDetailsList.addAll(orderVo.getOrderDetailsList());
            System.out.println("从 Redis 读取订单 ID：" + orders.getId());
        }

        String ordersData = importOrdersData(ordersList, true);
        String orderDetailsData = importOrderDetailsData(orderDetailsList, true);
        System.out.println("将缓存中的数据转移到数据库成功！");
        System.out.println("订单数据：" + ordersData);
        System.out.println("订单详情数据：" + orderDetailsData);
        System.out.println("订单列表大小：" + ordersList.size());
        System.out.println("订单详情列表大小：" + orderDetailsList.size());

        resetOrdersCache();
    }

    /**
     * 从 namespaceData 中获取 OrderVo 对象
     */
    private MOrdersVo getOrderVoFromNamespaceData(Map<String, Object> namespaceData, String key) {
        Object value = namespaceData.get(key);
        if (value == null) {
            return null;
        }
        if (value instanceof JSONObject) {
            JSONObject jsonObject = (JSONObject) value;

            return JSON.parseObject(jsonObject.toJSONString(), MOrdersVo.class);
        } else {
            throw new ClassCastException("期望 JSONObject 类型，但找到 " + value.getClass().getName());
        }
    }

    /**
     * 导入Order数据
     *
     * @param dataList 数据列表
     * @return 结果
     */
    public String importOrdersData(List<MOrders> dataList, boolean updateSupport) {
        MBulkImportService<MOrders> importService = new MBulkImportService<MOrders>() {

            @Override
            protected Long checkExistingItem(MOrders item) {
                MOrders orders = ordersMapper.selectOrdersById(item.getId());
                return orders == null ? 0L : orders.getId();
            }

            @Override
            protected boolean updateItem(MOrders item, Long id) {
                item.setId(id);
                return ordersMapper.updateOrders(item) > 0;
            }

            @Override
            protected boolean executeBatch(List<MOrders> batchDataList) {
                return ordersMapper.insertOrdersDataList(batchDataList) > 0;
            }

            @Override
            protected String getItemTitle(MOrders item) {
                return item.getId().toString();
            }

     };
        return importService.importData(dataList, updateSupport);
    }

    /**
     * 导入OrderDetails数据
     *
     * @param dataList 数据列表
     * @return 结果
     */
    public String importOrderDetailsData(List<MOrderDetails> dataList, boolean updateSupport) {
        MBulkImportService<MOrderDetails> importService = new MBulkImportService<MOrderDetails>() {

            @Override
            protected Long checkExistingItem(MOrderDetails item) {
                MOrderDetails orderDetails = orderDetailsMapper.selectOrderDetailsById(item.getId());
                return orderDetails == null ? 0L : orderDetails.getId();
            }

            @Override
            protected boolean updateItem(MOrderDetails item, Long id) {
                item.setId(id);
                return orderDetailsMapper.updateOrderDetails(item) > 0;
            }

            @Override
            protected boolean executeBatch(List<MOrderDetails> batchDataList) {
                return orderDetailsMapper.insertOrderDetailsDataList(batchDataList) > 0;
            }

            @Override
            protected String getItemTitle(MOrderDetails item) {
                return item.getId().toString();
            }
        };

        return importService.importData(dataList, updateSupport);
    }

    /**
     * 支付订单
     */
    @Override
    @Transactional
    public AjaxResult payment(Long userId, Long orderId) {
        // 获取订单详情
        MOrdersVo orderVo = orderCacheService.selectOrderVoToCache(userId, orderId);
        // 获取用户
        MUser vrUser = userMapper.selectUserById(userId);
        // 判断用户余额是否充足
        BigDecimal balance = vrUser.getMoney().subtract(orderVo.getTotalAmount());
        if (vrUser.getMoney().compareTo(orderVo.getTotalAmount()) < 0) {
            return AjaxResult.error("余额不足");
        }

        // 创建支付记录
        Long transactionsId = paymentLogControls(userId,orderId, orderVo.getTotalAmount(), balance);
        if (transactionsId.equals(0L)) {
            return AjaxResult.error("支付失败");
        }

        // 用户扣款
        if (!feeDeductionControls(vrUser, balance, transactionsId)) {
            return AjaxResult.error("扣款失败");
        }

        // 添加模板与用户相关记录
        int successNum = 0;    // 成功消息数
        int failureNum = 0;    // 失败消息数
        // 失败消息详情
        final StringBuilder failureMsg = new StringBuilder();
        List<MOrderDetails> orderDetailsList = orderVo.getOrderDetailsList();
        for (MOrderDetails orderDetails : orderDetailsList) {
            // 添加记录
            if (!addTemplateLogControls(userId, orderDetails)) {
                // 返还未购买成功的模板金额
                String result = returnTemplateAmount(orderDetails, vrUser);
                failureNum++;
                failureMsg.append(result).append("\n");
                continue;
            }
            successNum++;
        }

        // 修改订单状态
        orderCacheService.updateOrderStatusToCache(orderCacheService.getCacheKey(userId, orderId), 2);

        if (successNum == 0) {
            return AjaxResult.error("支付失败", failureMsg.toString());
        }
        if (failureNum > 0) {
            return AjaxResult.error("部分订单支付失败", failureMsg.toString());
        }
        return AjaxResult.success("支付成功");
    }

    /**
     * 创建支付记录相关操作
     */
    public Long paymentLogControls(Long userId,Long orderId, BigDecimal recharge, BigDecimal balance) {
        // 添加支付记录
        MTransactions transactions = new MTransactions();
        transactions.setUserId(userId);
        transactions.setRecharge(recharge);
        transactions.setMoney(balance);
        transactions.setPaymentMethod(4);
        transactions.setPaymentType(1);
        transactions.setStatus(1);
        transactions.setOrderId(orderId);
        if (transactionsMapper.insertTransactions(transactions) < 1) {
            return 0L;
        }

        return transactions.getId();
    }

    /**
     * 用户扣款相关操作
     */
    public boolean feeDeductionControls(MUser vrUser, BigDecimal balance, Long transactionId) {
        // 更新用户余额
        vrUser.setMoney(balance);
        vrUser.setUpdateTime(DateUtils.getNowDate());
        MTransactions transactions = transactionsMapper.selectTransactionsById(transactionId);

        if (userMapper.updateVrUser(vrUser) < 1) {
            transactions.setStatus(2);
        }else {
            transactions.setStatus(0);
        }

        return transactionsMapper.updateTransactions(transactions) > 0;
    }

    /**
     * 添加模板与用户相关记录相关操作
     */
    public boolean addTemplateLogControls(Long userId, MOrderDetails orderDetails) {
        MTemplateLog templateLog = new MTemplateLog();
        templateLog.setTemplateId(orderDetails.getTemplateId());
        templateLog.setUserId(userId);
        templateLog.setCreateTime(DateUtils.getNowDate());
        return templateLogMapper.insertTemplateLog(templateLog) > 0;
    }

    /**
     * 返还模板金额
     */
    public String returnTemplateAmount(MOrderDetails orderDetails, MUser vrUser) {
        String baseStr = "模板id:" + orderDetails.getTemplateId();
        // 返还未购买成功的模板金额
        BigDecimal balance = vrUser.getMoney().add(orderDetails.getPrice());
        vrUser.setMoney(balance);
        vrUser.setUpdateTime(DateUtils.getNowDate());
        if (userMapper.updateVrUser(vrUser) < 1) {
            return "返还用户余额失败;" + baseStr;
        }
        // 添加支付记录
        MTransactions t = new MTransactions();
        t.setUserId(vrUser.getId());
        t.setRecharge(orderDetails.getPrice());
        t.setMoney(balance);
        t.setPaymentMethod(4);
        t.setPaymentType(2);
        t.setStatus(1);
        t.setMark("添加模板记录失败,返还金额");
        if (transactionsMapper.insertTransactions(t) < 1) {
            return "返回余额成功，创建支付记录失败;" + baseStr;
        }
        return "返还模板金额成功" + baseStr;
    }

    /**
     * 添加模板名称
     */
    public MOrdersVo fillTemplateNames(MOrdersVo orderVo) {
        for (MOrderDetails detail : orderVo.getOrderDetailsList()) {
            MTemplate template = templateService.selectTemplateById(detail.getTemplateId());
            String templateName = (template != null) ? template.getTitle() : "未知模板";
            detail.setTemplateName(templateName);

            String templateImgUrl = (template != null) ? template.getImgUrl() : "图片未找到";
            detail.setImgUrl(templateImgUrl);
        }
        return orderVo;
    }

}

